# Copyright 2009 Neil Martinsen-Burrell

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


from breezy.plugins.colo.colocated import (
    COLOCATED_LOCATION,
    ColocatedDirectory,
    ColocatedWorkspace,
    NoColocatedWorkspace,
    colo_nick_hook,
    )
from breezy.plugins.colo.commands import (init_workspace,
                                          COLOCATED_STAGING,
                                         )
from breezy.plugins.colo.tests.script import ScriptTestCase

from breezy.lazy_import import lazy_import
lazy_import(globals(), '''
from breezy import (branch,
                    directory_service,
                    errors,
                    tests,
                    osutils,
                    urlutils,
                   )
from breezy.bzr import bzrdir
''')


def make_colo_branch(workspace, name):
    transport = workspace.repo_transport().clone(name)
    transport.create_prefix()
    return bzrdir.BzrDir.create_branch_convenience(transport.base)


class TestColoLookup(tests.TestCaseWithTransport):

    def test_colo_lookup_fail(self):
        """Look up a colo: URL while not in a colocated workspace"""
        directory = ColocatedDirectory()
        self.assertRaises(NoColocatedWorkspace,
                          directory.look_up, '', 'colo:')

    def test_colo_lookup_nonexistent(self):
        """Look up a nonexistent branch under colo:"""
        directory = ColocatedDirectory()
        init_workspace(self.test_dir, outf=tests.StringIOWrapper())
        self.assertEqual(directory.look_up('test', 'colo:test'),
                         urlutils.local_path_to_url(osutils.pathjoin(
                             self.test_dir, COLOCATED_LOCATION, 'test')))

    def test_colo_in_directory_services(self):
        """Look up a branch via directory services."""
        self.assertRaises(NoColocatedWorkspace,
                          directory_service.directories.dereference, 
                          'colo:test')

    def test_colo_remote_error(self):
        self.assertRaises(NoColocatedWorkspace,
                          directory_service.directories.dereference,
                          'colo:test:remote')

    def test_colo_remote(self):
        self.run_bzr('colo-init test')
        self.assertEqual(directory_service.directories.dereference(
                         'colo:test:remote'),
                          urlutils.join(self.get_url(), 'test', COLOCATED_LOCATION, 'remote'))

    def test_error_code(self):
        self.run_bzr_error(['No colocated workspace'],
                           ['colo-branches'])

    def test_backslash_name(self):
        """A name with a backslash should be converted to a forward slash"""
        self.run_bzr('colo-init')
        self.assertEqual(directory_service.directories.dereference(
                         r'colo:test\backslash'),
                         urlutils.join(self.get_url(), COLOCATED_LOCATION,  'test/backslash'))



class TestColocatedWorkspace(tests.TestCaseInTempDir):

    def test_creation(self):
        init_workspace('.', outf=tests.StringIOWrapper())
        z = ColocatedWorkspace()
        self.assertStartsWith(z.base,
                              urlutils.local_path_to_url(self.test_dir))

    def test_creation_directory(self):
        init_workspace('branch', outf=tests.StringIOWrapper())
        z = ColocatedWorkspace('branch')
        self.assertEndsWith(z.base, 'branch/')

    def test_repo_location(self):
        init_workspace('.', outf=tests.StringIOWrapper())
        z = ColocatedWorkspace()
        self.assertEqual(z.repo_location, 
                         urlutils.local_path_to_url(
                             urlutils.join(self.test_dir, COLOCATED_LOCATION))
                        )

    def test_remote_no_tree_repo_location(self):
        init_workspace('.', create_tree=False, outf=tests.StringIOWrapper())
        z = ColocatedWorkspace(urlutils.normalize_url(self.test_dir))
        self.assertEqual(z.repo_location,
                         urlutils.normalize_url(
                             urlutils.join(self.test_dir, COLOCATED_LOCATION))
                        )

    def test_branch_names(self):
        init_workspace('.', outf=tests.StringIOWrapper())
        z = ColocatedWorkspace()
        self.assertEqual(list(z.branch_names()), ['trunk'])

    def test_current_branch(self):
        init_workspace('.', outf=tests.StringIOWrapper())
        z = ColocatedWorkspace()
        self.assertEndsWith(z.current_branch().base, 'trunk/')

    def test_current_branch_name(self):
        init_workspace('.', outf=tests.StringIOWrapper())
        z = ColocatedWorkspace()
        self.assertEqual(z.current_branch_name(), 'trunk')

    def test_is_pipe_non_existent(self):
        init_workspace('.', outf=tests.StringIOWrapper())
        z = ColocatedWorkspace()
        self.assertFalse(z.is_pipe('non_existent'))

    def test_is_pipe_plain_branch(self):
        init_workspace('.', outf=tests.StringIOWrapper())
        z = ColocatedWorkspace()
        branch = make_colo_branch(z, 'has_next')
        self.assertFalse(z.is_pipe('has_next'))

    def test_is_pipe_has_next(self):
        init_workspace('.', outf=tests.StringIOWrapper())
        z = ColocatedWorkspace()
        branch = make_colo_branch(z, 'has_next')
        branch.get_config().set_user_option('next_pipe', 'foo')
        self.assertTrue(z.is_pipe('has_next'))

    def test_is_pipe_has_blank_next(self):
        init_workspace('.', outf=tests.StringIOWrapper())
        z = ColocatedWorkspace()
        branch = make_colo_branch(z, 'has_next')
        branch.get_config().set_user_option('next_pipe', '')
        self.assertFalse(z.is_pipe('has_next'))

    def test_is_pipe_has_prev(self):
        init_workspace('.', outf=tests.StringIOWrapper())
        z = ColocatedWorkspace()
        branch = make_colo_branch(z, 'has_prev')
        branch.get_config().set_user_option('prev_pipe', 'foo')
        self.assertTrue(z.is_pipe('has_prev'))

    def test_is_pipe_has_blank_prev(self):
        init_workspace('.', outf=tests.StringIOWrapper())
        z = ColocatedWorkspace()
        branch = make_colo_branch(z, 'has_prev')
        branch.get_config().set_user_option('prev_pipe', '')
        self.assertFalse(z.is_pipe('has_prev'))


class TestColoScripts(ScriptTestCase):

    def test_colo_blackbox(self):
        """colo: from the command line"""
        self.run_script("""
$ brz colo-init
$ brz init colo:test
$ cat .bzr/branches/test/.bzr/README
""")

    def test_colo_remote_blackbox(self):
        self.run_script("""
$ brz colo-init b1
$ brz colo-init b2
$ cd b1
$ brz log colo:../b2:trunk
""")

    def test_colo_remote_url(self):
        script_text = '\n'.join(["$ brz colo-init",
                                 "$ brz log colo:%s:trunk" % self.get_url(),
                                    ])
        self.run_script(script_text)

    def test_colo_init(self):
        """Creating a colocated branch using the alias."""
        self.run_script("""
$ brz colo-init branch
$ cat branch/.bzr/README
$ cat branch/.bzr/branches/.bzr/README
$ cat branch/.bzr/branches/trunk/.bzr/README
""")

    def test_branches(self):
        self.run_script("""
$ brz colo-init
$ brz colo-branch fix1
$ brz colo-branch --no-switch fix2
""")
        z = ColocatedWorkspace()
        self.assertEqual(len(list(z.branches())), 3)

    def test_hierarchical_names(self):
        self.run_script("""
$ brz colo-init
$ brz colo-branch a/b
$ brz colo-branches
* a/b
  trunk
""")
        z = ColocatedWorkspace()
        self.assertEqual(z.current_branch_name(), 'a/b')

    def test_backslash_names(self):
        self.run_script("""
$ brz colo-init
$ brz colo-branch trunk\\test
$ brz colo-branches
  trunk
* trunk/test
$ brz log colo:trunk\\test
""")

    def test_backslash_bialix(self):
        self.run_script("""
$ brz colo-init
$ brz ci -m1 --unchanged
$ brz switch -b colo:trunk\\bar
$ brz colo-branches
  trunk
* trunk/bar
$ brz ci -m2 --unchanged
$ brz switch colo:trunk
$ brz merge colo:trunk\\bar
""")

    def test_colo_init_hidden(self):
        out, err = self.run_bzr(['colo-init'])
        self.assertNotContainsRe(out,
                                 COLOCATED_STAGING)

    def test_colo_init_create_prefix(self):
        self.run_script("""
$ brz colo-init --create-prefix branch/test/long/path
$ cat branch/test/long/path/.bzr/README
""")

    def test_colo_init_trunk_name(self):
        self.run_script("""
$ brz colo-init --trunk-name=HEAD
$ brz colo-branches
* HEAD
""")

    def test_colo_init_trunk_name_hierarchical(self):
        self.run_script("""
$ brz colo-init --trunk-name=origin/trunk
$ brz colo-branches
* origin/trunk
""")

    def test_colo_init_no_tree(self):
        self.run_script("""
$ brz colo-init --no-tree
$ brz colo-branches
  trunk
""")

    def test_colo_checkin(self):
        """Use a colocated branch to check in under trunk."""
        self.run_script("""
$ brz colo-init
$ cat > file1
<text
$ cat < file1
text
$ brz add
$ brz ci -m 'first check-in'
2>Committing to: .../.bzr/branches/trunk/
2>added file1
2>Committed revision 1.
$ brz log -r1
------------------------------------------------------------
revno: 1
committer: ...
branch nick: trunk
timestamp: ...
message:
  first check-in
""")

    def test_colo_switch_creates_branch(self):
        """Branch using the switch command in a colocated tree"""
        self.run_script("""
$ brz colo-init
$ brz switch -b newtip
$ cat .bzr/branches/newtip/.bzr/README
$ brz nick
newtip
""")

    def test_colo_switch_creates_branch_use_directory_service(self):
        """Branch using the switch command in a colocated tree"""
        self.run_script("""
$ brz colo-init
$ brz switch -b colo:newtip/  # slash for workaround on switch bug
$ cat .bzr/branches/newtip/.bzr/README
$ brz nick
newtip
""")

    def test_colo_merging(self):
        """Merge in colocated branches"""
        self.run_script("""
$ brz colo-init
$ cat > file1
<text
$ brz add
$ brz ci -m 'adding first file'
$ brz switch -b newtip
$ brz nick
newtip
$ cat >> file1
<more text
$ cat < file1
text
more text
$ brz ci -m 'change in newtip'
2>Committing to: .../.bzr/branches/newtip/
2>modified file1
2>Committed revision 2.
$ brz switch trunk
$ brz merge colo:newtip
$ brz ci -m 'merge in changes from newtip'
2>Committing to: .../.bzr/branches/trunk/
2>modified file1
2>Committed revision 2.
$ brz log --line -n0
2: ... [merge] merge in changes from newtip
  1.1.1: ... change in newtip
1: ... adding first file
""")

class TestColoBranch(ScriptTestCase):

    def test_colo_branch_then_switch(self):
        """Branch in a colocated tree."""
        self.run_script("""
$ brz colo-init
$ brz branch . colo:newtip
$ brz switch newtip  # works because newtip is a sibling of trunk
$ brz nick
newtip
""")

    def test_colo_branch(self):
        self.run_script("""
$ brz colo-init
$ brz colo-branch newtip
2>Branched 0 revisions.
2>Tree is up to date at revision 0.
2>Switched to branch: .../.bzr/branches/newtip/
$ brz nick
newtip
""")

    def test_colo_branch_no_switch(self):
        self.run_script("""
$ brz colo-init
$ brz colo-branch --no-switch newtip
2>Branched 0 revisions.
$ brz nick
trunk
""")

    def test_colo_branch_from(self):
        self.run_script("""
$ brz colo-init
$ brz ci -m 'commit in trunk' --unchanged
$ brz colo-branch new
$ brz ci -m 'commit in new branch' --unchanged
$ brz colo-branch --from-branch=trunk new2
$ brz log --line
1: ...
""")

    def test_colo_branch_revision(self):
        self.run_script("""
$ brz colo-init
$ cat >file1
<content
$ brz add
$ brz ci -m 'first'
$ cat >>file1
<more content
$ brz ci -m 'second'
$ brz colo-branch -r1 new
$ cat file1
content
""")


class TestColoBranches(ScriptTestCase):

    def test_colo_branches(self):
        self.run_script("""
$ brz colo-init
$ brz colo-branch new
$ brz colo-branch newer
$ brz colo-branches
  new
* newer
  trunk
""")

    def test_colo_branches_location(self):
        self.run_script("""
$ brz colo-init test
$ cd test
$ brz colo-branch new
$ cd ..
$ brz colo-branches test
  new
  trunk
""")

    def test_colo_branches_hidden(self):
        self.run_script("""
$ brz colo-init
$ brz colo-branch --no-switch .test
$ brz colo-branches
* trunk
""")

    def test_colo_branches_config_hidden(self):
        self.run_script("""
$ brz colo-init
$ brz colo-branch test
$ brz colo-branches
* test
  trunk
$ brz config bzr.branch.status=hidden
$ brz colo-branches
* test
  trunk
$ brz switch trunk
$ brz colo-branches
* trunk
$ brz colo-branches --all
  test
* trunk
$ brz switch test
$ brz config bzr.branch.status=active
$ brz colo-branches
* test
  trunk
$ brz switch trunk
$ brz colo-branches
  test
* trunk
""")

    def test_colo_branches_hidden_config_other_values(self):
        init_workspace('.', outf=tests.StringIOWrapper())
        z = ColocatedWorkspace()
        branch = make_colo_branch(z, 'test')
        for hidden_value in ColocatedWorkspace.HIDDEN_STATUSES:
            self.run_script("""
$ brz switch test
$ brz config bzr.branch.status=%s
$ brz switch trunk
$ brz colo-branches
* trunk
""" % hidden_value)

    def test_colo_branches_all(self):
        self.run_script("""
$ brz colo-init
$ brz colo-branch --no-switch .test
$ brz colo-branches --all
  .test
* trunk
""")

    def test_colo_branches_current_not_hidden(self):
        self.run_script("""
$ brz colo-init
$ brz colo-branch --no-switch .other
$ brz colo-branch .this
$ brz colo-branches 
* .this
  trunk
$ brz colo-branches --all
  .other
* .this
  trunk
""")

    def test_colo_branches_hidden_subdirectory(self):
        self.run_script("""
$ brz colo-init
$ brz colo-branch --no-switch other
$ brz colo-branch --no-switch this
$ brz colo-branches
  other
  this
* trunk
$ brz colo-mv other .hidden/other
$ brz colo-mv this .hidden/this
$ brz colo-branches
* trunk
$ brz colo-branches --all
  .hidden/other
  .hidden/this
* trunk
""")

    def test_colo_branches_hidden_in_subdirectory(self):
        self.run_script("""
$ brz colo-init
$ brz colo-branch --no-switch subdir/.this
$ brz colo-branch --no-switch foo/.bar/spam
$ brz colo-branches
* trunk
$ brz colo-branches --all
  foo/.bar/spam
  subdir/.this
* trunk
""")


class TestColoFetch(ScriptTestCase):

    def test_fetch(self):
        self.make_branch('branch')
        self.run_script("""
$ brz colo-fetch branch branch-colo
$ cd branch-colo
$ brz colo-branches
* origin/trunk
""")

    def test_fetch_current_directory(self):
        self.make_branch('branch')
        self.run_script("""
$ brz colo-fetch branch
2>brz: ERROR: A control directory already exists: .../branch/...
""")

    def test_fetch_one_argument(self):
        self.run_script("""
$ brz init --create-prefix other/branch
$ brz colo-fetch other/branch
$ brz colo-branches branch
  origin/trunk
""")

    def test_fetch_url(self):
        self.make_branch('branch')
        branch_url = urlutils.local_path_to_url('branch')
        self.run_script(
            ''.join(["$ brz colo-fetch %s .\n" % branch_url,
                     "$ brz colo-branches\n",
                     "* origin/trunk"]))

    def test_fetch_branch_name(self):
        self.make_branch('branch')
        self.run_script("""
$ brz colo-fetch --trunk-name=master branch .
$ brz colo-branches
* master
""")


class TestColoCheckout(ScriptTestCase):

    def test_checkout(self):
        self.run_script("""
$ brz colo-init branch
$ cd branch
$ brz colo-checkout ../branch2
$ cd ../branch2
$ cat .bzr/README
""")

    def test_checkout_example(self):
        self.run_script("""
$ brz colo-init branch
$ cd branch
$ brz colo-branch fix-eol-bug
$ brz colo-branches
* fix-eol-bug
  trunk
$ brz switch trunk
$ brz colo-branch fix-unicode-normalization
$ brz colo-checkout ../fix-unicode-normalization
$ cd ../fix-unicode-normalization
$ cat .bzr/README
""")

    def test_checkout_create(self):
        self.run_script("""
$ brz colo-init branch
$ cd branch
$ brz colo-checkout --create ../fix1
$ brz colo-branches
  fix1
* trunk
$ brz nick
trunk
$ cd ../fix1
$ cat .bzr/README
$ brz nick
fix1
$ brz colo-branches
* fix1
  trunk
""")

    def test_checkout_create_dash_b(self):
        self.run_script("""
$ brz colo-init branch
$ cd branch
$ brz colo-checkout -b ../fix1
$ brz colo-branches
  fix1
* trunk
$ brz nick
trunk
$ cd ../fix1
$ cat .bzr/README
$ brz nick
fix1
$ brz colo-branches
* fix1
  trunk
""")

    def test_checkout_in_directory(self):
        self.run_script("""
$ brz colo-init
$ brz colo-checkout tree
$ cd tree
$ cat .bzr/README
$ brz nick
trunk
""")

    def test_checkout_from_branch(self):
        self.run_script("""
$ brz colo-init branch
$ cd branch
$ brz colo-branch fix-eol-bug
$ brz colo-branches
* fix-eol-bug
  trunk
$ brz colo-checkout --from-branch trunk ../fix-unicode-normalization
$ cd ../fix-unicode-normalization
$ brz nick
trunk
$ brz colo-branches
  fix-eol-bug
* trunk
""")


class TestColoPrune(ScriptTestCase):

    def test_prune(self):
        self.run_script("""
$ brz colo-init
$ brz colo-branch new
$ brz colo-branches
* new
  trunk
$ brz switch trunk
$ brz colo-branches
  new
* trunk
$ brz colo-prune new
$ brz colo-branches
* trunk
""")

    def test_prune_multiple(self):
        self.run_script("""
$ brz colo-init
$ brz colo-branch --no-switch new
$ brz colo-branch --no-switch newer
$ brz colo-prune new newer
$ brz colo-branches
* trunk
""")

    def test_prune_clean(self):
        self.run_script("""
$ brz colo-init
$ brz ci -m 'trunk revision' --unchanged
$ brz colo-branch new
$ brz ci -m 'a revision' --unchanged
$ brz switch trunk
$ brz colo-prune --clean new
$ brz colo-branches
* trunk
""")
        repo = ColocatedWorkspace().repository()
        self.assertEqual(len(repo.all_revision_ids()), 1)

    def test_prune_current(self):
        self.run_script("""
$ brz colo-init
$ brz colo-prune trunk
2>Not removing current branch trunk.
""")

    def test_dont_prune_everything(self):
        self.run_script("""
$ brz colo-init
$ brz colo-branch one
$ brz colo-prune .
2>Not removing . because it is not a branch.
$ cat .bzr/branches/.bzr/README
""")

    def test_dont_prune_subdirectories(self):
        self.run_script("""
$ brz colo-init
$ brz colo-branch origin/trunk
$ brz colo-prune origin
2>Not removing origin because it is not a branch.
$ cat .bzr/branches/origin/trunk/.bzr/README
""")

    def test_do_prune_common_prefixes(self):
        self.run_script("""
$ brz colo-init
$ brz colo-branch other
$ brz colo-branch other-and-more
$ brz colo-prune other
""")

    def test_dont_prune_subdirectories_of_branch(self):
        self.run_script("""
$ brz colo-init
$ brz colo-branch --no-switch origin
$ brz colo-branch --no-switch origin/trunk
$ brz colo-prune origin
2>Not removing because of branches in subdirectories.
$ brz colo-branches
  origin
  origin/trunk
* trunk
""")

    def test_prune_colo_prefix(self):
        self.run_script("""
$ brz colo-init
$ brz colo-branch new
$ brz colo-prune colo:trunk
$ brz colo-branches
* new
""")

    def test_prune_pipe(self):
        init_workspace('.', outf=tests.StringIOWrapper())
        z = ColocatedWorkspace()
        branch = make_colo_branch(z, 'has_next')
        branch.get_config().set_user_option('next_pipe', 'foo')
        self.run_script("""
$ brz colo-prune colo:has_next
2>Not removing has_next because it is a pipe.  Use remove-pipe --branch.
$ brz colo-branches
  has_next
* trunk""")


class TestColoClean(ScriptTestCase):

    def test_clean(self):
        self.run_script("""
$ brz colo-init
$ brz ci -m 'trunk revision' --unchanged
$ brz colo-branch new
$ brz ci -m 'a revision' --unchanged
$ brz switch trunk
$ brz colo-prune new
$ brz colo-clean
""")
        repo = ColocatedWorkspace().repository()
        self.assertEqual(len(repo.all_revision_ids()), 1)

    def test_clean_early_branch(self):
        self.run_script("""
$ brz colo-init
$ brz ci -m 'first revision' --unchanged
$ brz ci -m 'second revision' --unchanged
$ brz colo-branch -r 1 new
$ brz ci -m 'new revision' --unchanged
$ brz switch trunk
$ brz colo-prune new
$ brz colo-clean
""")
        self.assertEqual(
            len(ColocatedWorkspace().repository().all_revision_ids()), 2)

    def test_clean_merge(self):
        self.run_script("""
$ brz colo-init
$ brz ci -m 'first revision' --unchanged
$ brz ci -m 'second revision' --unchanged
$ brz colo-branch -r 1 new
$ brz ci -m 'new revision' --unchanged
$ brz switch trunk
$ brz merge colo:new
$ brz ci -m 'merge revision'
$ brz colo-prune new
$ brz colo-clean
""")
        # clean shouldn't remove anything
        self.assertEqual(
            len(ColocatedWorkspace().repository().all_revision_ids()), 4)

class TestColoIfy(ScriptTestCase):

    def test_colo_ify(self):
        self.run_script("""
$ brz init test
$ cd test
$ brz colo-ify
""")

    def test_colo_ify_change_name(self):
        self.run_script("""
$ brz init test
$ cd test
$ brz colo-ify --trunk-name=HEAD
$ brz colo-branches
* HEAD
""")

    def test_colo_ify_others(self):
        self.run_script("""
$ brz init test
$ brz init test2
$ cd test
$ brz colo-ify ../test2
$ brz colo-branches
  test2
* trunk
""")

    def test_colo_ify_check_no_parent(self):
        self.run_script("""
$ brz init test
$ cd test
$ brz colo-ify
""")
        out, err = self.run_bzr(['info'])
        self.assertNotContainsRe(out, 'parent')

    def test_colo_ify_check_parent(self):
        self.run_script("""
$ brz init test
$ brz branch test test2
$ brz branch test2 test3
$ cd test
$ brz colo-ify ../test2 ../test3
$ brz info colo:test3
...
  parent branch: .../test2
""")

    def test_colo_ify_bound(self):
        self.run_script("""
                        $ brz init a
                        $ brz co a b
                        $ cd b
                        $ brz colo-ify
                        $ brz info
                        """)
        out, err = self.run_bzr(['info'])
        self.assertContainsRe(out, 'checkout of branch')

class TestColoFixup(ScriptTestCase):

    def test_fixup_is_needed(self):
        self.run_script("""
$ brz colo-init test
$ mv test test2
$ cd test2
""")
        self.run_bzr(['colo-branches'], retcode=3, 
                     error_regexes=['Not a branch:'])

    def test_fixup(self):
        self.run_script("""
$ brz colo-init test
$ mv test test2
$ cd test2
$ brz colo-fixup
$ brz colo-branches
* trunk
""")

    def test_fixup_multiple(self):
        self.run_script("""
$ brz colo-init test
$ cd test
$ brz colo-branch feature
$ cd ..
$ mv test test2
$ cd test2
$ brz colo-fixup
$ brz colo-branches
* feature
  trunk
""")

    def test_fixup_hierarchical(self):
        self.run_script("""
$ brz colo-init test
$ cd test
$ brz colo-branch a/b
$ cd ..
$ mv test test2
$ cd test2
$ brz colo-fixup
$ brz colo-branches
* a/b
  trunk
""")

    def test_fixup_hierarchical_from_trunk(self):
        self.run_script("""
$ brz colo-init test
$ cd test
$ brz colo-branch a/b
$ brz switch colo:trunk
$ cd ..
$ mv test test2
$ cd test2
$ brz colo-fixup
$ brz colo-branches
  a/b
* trunk
""")

    def test_fixup_from_subdir(self):
        self.run_script("""
$ brz colo-init test
$ cd test
$ mkdir subdir
$ cd ..
$ mv test test2
$ cd test2/subdir
$ brz colo-fixup
$ brz colo-branches
* trunk
""")


class TestHierarchicalNames(ScriptTestCase):

    def test_branch(self):
        self.run_script("""
$ brz colo-init
$ brz colo-branch a/b
$ brz colo-branch a/c
$ brz colo-branch a/c/d
""")

    def test_branches(self):
        self.run_script("""
$ brz colo-init
$ brz colo-branch a/b
$ brz colo-branches
* a/b
  trunk
""")

    def test_checkout(self):
        self.run_script("""
$ brz colo-init test
$ cd test
$ brz colo-branch a/b
$ brz colo-checkout ../b-checkout
$ cd ../b-checkout
$ brz switch colo:trunk
""")

    def test_prune(self):
        self.run_script("""
$ brz colo-init
$ brz colo-branch a/b
$ brz switch colo:trunk
$ brz colo-prune a/b
$ brz colo-branches
* trunk
""")


class TestColoPull(ScriptTestCase):

    def test_pull(self):
        self.run_script("""
$ brz colo-init workspace
$ cd workspace
$ brz colo-branch origin/test
$ brz colo-pull
2>Updating origin/test from .../trunk/
No revisions or tags to pull.
""")

    def test_pull_outside_branch(self):
        self.run_script("""
$ brz init branch
$ brz colo-init workspace
$ cd workspace
$ brz colo-branch origin/test
$ brz pull --remember ../branch
$ brz colo-pull
2>Updating origin/test from .../branch/
No revisions or tags to pull.
""")

    def test_pull_revision(self):
        self.run_script("""
$ brz init branch
$ brz colo-init workspace
$ cd workspace
$ brz colo-branch origin/test
$ brz pull --remember ../branch
$ cd ../branch
$ echo "contents" > file
$ brz add
$ brz ci -m 'first'
$ cd ../workspace
$ brz colo-pull
Now on revision 1.
""")

    def test_pull_diverged(self):
        self.run_script("""
$ brz init branch
$ brz colo-init workspace
$ cd workspace
$ brz colo-branch origin/test
$ brz pull --remember ../branch
$ cd ../branch
$ echo "contents" > file
$ brz add
$ brz ci -m 'first'
$ cd ../workspace
$ echo "other contents" > file2
$ brz add
$ brz ci -m 'a new revision'
""")
        out, err = self.run_bzr('colo-pull', retcode=3)
        self.assertEqual(out, '')
        self.assertContainsRe(err, 'brz: ERROR: These branches have diverged.'
                              ' Use the missing command to see how.\n'
                              'Use the merge command to reconcile them.\n')

    def test_pull_multiple(self):
        self.run_script("""
$ brz init branch
$ brz colo-init workspace
$ cd workspace
$ brz colo-branch origin/test
$ brz pull --remember ../branch
$ brz colo-branch origin/other
$ brz pull --remember ../branch
$ cd ../branch
$ echo "contents" > file
$ brz add
$ brz ci -m 'first'
$ cd ../workspace
$ brz colo-pull
2>Updating origin/other from .../branch/
2>All changes applied successfully.
2>Updating origin/test from .../branch/
Now on revision 1.
Now on revision 1.
""")

    def test_pull_current(self):
        self.run_script("""
$ brz init branch
$ brz colo-init workspace
$ cd workspace
$ brz colo-mv trunk origin/trunk
$ brz pull --remember ../branch
$ cd ../branch
$ echo content > file
$ brz add
$ brz ci -m 'a revision on branch'
$ cd ../workspace
$ brz colo-pull
Now on revision 1.
$ cat file
content
""")

    def test_pull_subdirectory(self):
        self.run_script("""
$ brz init branch
$ brz colo-init workspace
$ cd workspace
$ brz pull --remember ../branch
$ mkdir subdir
$ cd subdir
$ brz colo-pull
""")
                        


class TestColoMv(ScriptTestCase):

    def test_mv(self):
        self.run_script("""
$ brz colo-init
$ brz colo-branches
* trunk
$ brz colo-mv trunk test
$ brz colo-branches
* test
""")

    def test_mv_hierarchical(self):
        self.run_script("""
$ brz colo-init
$ brz colo-branches
* trunk
$ brz colo-mv trunk origin/trunk
$ brz colo-branches
* origin/trunk
""")

    def test_mv_hierarchical_existing(self):
        self.run_script("""
$ brz colo-init
$ brz colo-branch origin/test
$ brz colo-mv trunk origin/trunk
""")

    def test_mv_not_current(self):
        self.run_script("""
$ brz colo-init
$ brz colo-branch new
$ brz colo-mv trunk HEAD
$ brz colo-branches
  HEAD
* new
""")


class TestNoTree(ScriptTestCase):

    def test_list_branches(self):
        self.run_script("""
$ brz colo-init --no-tree
$ brz colo-branches
  trunk
""")

    def test_branch_current(self):
        self.run_script("""
$ brz colo-init --no-tree
$ brz colo-branch new
2>brz: ERROR: Cannot find a current branch in a no-tree colocated workspace.
""")

    def test_branch_from(self):
        self.run_script("""
$ brz colo-init --no-tree
$ brz colo-branch --from=trunk new
$ brz colo-branches
  new
  trunk
""")

    def test_pull(self):
        self.run_script("""
$ brz colo-init remote
$ brz colo-init --no-tree --trunk-name=origin/trunk local
$ cd local
$ brz branch colo:../remote:trunk colo:origin/remote
$ brz colo-pull
2>Updating origin/remote from .../remote/.bzr/branches/trunk/
""")

    def test_checkout(self):
        self.run_script("""
$ brz colo-init --no-tree branch
$ cd branch
$ brz colo-checkout ../checkout
2>brz: ERROR: Cannot find a current branch in a no-tree colocated workspace.
""")

    def test_prune(self):
        self.run_script("""
$ brz colo-init --no-tree
$ brz colo-branch --from=trunk new
$ brz colo-prune new
""")

    def test_mv(self):
        self.run_script("""
$ brz colo-init --no-tree
$ brz colo-mv trunk HEAD
""")

    def test_clean(self):
        self.run_script("""
$ brz colo-init --no-tree
$ brz colo-clean
""")

    def test_fixup(self):
        self.run_script("""
$ brz colo-init --no-tree first
$ mv first second
$ cd second
$ brz colo-fixup
2>brz: ERROR: Not a branch: ...
""")

class TestNick(ScriptTestCase):

    def setUp(self):
        ScriptTestCase.setUp(self)
        branch.Branch.hooks.install_named_hook('post_branch_init',
            colo_nick_hook, 'colocated branch name nick')
        
    def test_no_dir_in_nick(self):
        self.run_script("""
$ brz config --scope=breezy colo.dir_in_nick=False
$ brz hooks
...
  post_branch_init:
    colocated branch name nick
...
$ brz colo-init test
$ cd test
$ brz nick
trunk
$ brz colo-branch bug/long-name
$ brz nick
bug/long-name
""")

    def test_dir_in_nick(self):
        self.run_script("""
$ brz config --scope=breezy colo.dir_in_nick=True
$ brz colo-init test
$ cd test
$ brz nick
test/trunk
$ brz switch -b feature
$ brz nick
test/feature
$ brz colo-branch subdir/feature
$ brz nick
test/subdir/feature
""")
