# Copyright 2010 Neil Martinsen-Burrell

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

from breezy.plugins.colo.tests.script import ScriptTestCase
from breezy.plugins.colo.colocated import ColocatedWorkspace


class TestColoSyncFrom(ScriptTestCase):

    def test_sync(self):
        self.run_script("""
$ brz colo-init remote
$ brz colo-init local
$ cd local
$ brz colo-sync-from ../remote
2>Updating branch trunk from colo:../remote:trunk.
No revisions or tags to pull.
""")

    def test_sync_nontrivial(self):
        self.run_script("""
$ brz colo-init remote
$ brz colo-init local
$ cd remote
$ echo content > file
$ brz add
$ brz ci -m 'a revision'
$ cd ../local
$ brz colo-sync-from ../remote
2>Updating branch trunk from colo:../remote:trunk.
2>All changes applied successfully.
Now on revision 1.
""")

    def test_sync_missing_branch(self):
        self.run_script("""
$ brz colo-init remote
$ brz colo-init local
$ cd local
$ brz colo-branch other
$ brz colo-sync-from ../remote
$ brz colo-branches
* other
  trunk
""")

    def test_sync_creates_branch(self):
        self.run_script("""
$ brz colo-init remote
$ brz colo-init local
$ cd remote
$ brz colo-branch other
$ cd ../local
$ brz colo-sync-from ../remote
$ brz colo-branches
  other
* trunk
""")


    def test_sync_diverged(self):
        self.run_script("""
$ brz colo-init remote
$ brz colo-init local
$ cd remote
$ echo content > file
$ brz add
$ brz ci -m 'a revision in remote branch'
$ cd ../local
$ echo stuff > file1
$ brz add
$ brz ci -m 'a different revision in local branch'
""")
        self.run_bzr_error(['These branches have diverged. Use the missing command to see how.'],
                           ['colo-sync-from', '../remote'])

    def test_sync_multiple_diverged(self):
        self.run_script("""
$ brz colo-init local
$ brz colo-init remote
$ cd remote
$ echo content > file
$ brz add
$ brz ci -m 'trunk revision'
$ brz colo-branch other
$ cd ../local
$ echo stuff > file
$ brz add
$ brz ci -m 'a different trunk revision'
$ brz colo-sync-from ../remote
2>Creating new branch other from colo:../remote:other.
2>Branched 1 revision.
2>Updating branch trunk from colo:../remote:trunk.
2>brz: ERROR: These branches have diverged...
2>...
""")

    def test_sync_overwrite(self):
        self.run_script("""
$ brz colo-init remote
$ brz colo-init local
$ cd remote
$ echo content > file
$ brz add
$ brz ci -m 'a revision in remote branch'
$ cd ../local
$ echo stuff > file1
$ brz add
$ brz ci -m 'a different revision in local branch'
$ brz colo-sync-from --overwrite ../remote
2>Updating branch trunk from colo:../remote:trunk.
2>All changes applied successfully.
Now on revision 1.
""")

    def test_sync_from_empty(self):
        self.run_script("""
$ mkdir empty
$ brz colo-init local
$ cd local
$ brz colo-sync-from ../empty
2>brz: ERROR: No colocated workspace in ../empty
""")

    def test_sync_from_standalone(self):
        self.run_script("""
$ brz colo-init local
$ brz init standalone
$ cd local
$ brz colo-sync-to ../standalone
2>brz: ERROR: No colocated workspace in ../standalone
""")

    def test_sync_from_repository(self):
        self.run_script("""
$ brz colo-init local
$ brz init-repo repo
$ brz init repo/trunk
$ cd repo/trunk
$ echo content > file
$ brz add
$ brz ci -m 'a revision in repository branch'
$ cd ../../local
$ brz colo-sync-from --repository ../repo
2>Updating branch trunk from colo:../repo:trunk.
2>All changes applied successfully.
Now on revision 1.
""")

    def test_sync_from_repository_no_argument(self):
        self.run_script("""
$ brz colo-init local
$ brz init-repo repo
$ brz init repo/trunk
$ cd repo/trunk
$ echo content > file
$ brz add
$ brz ci -m 'a revision in repository branch'
$ cd ../../local
$ brz colo-sync-from ../repo
$ cat file
2>file: No such file or directory
""")

    def test_sync_from_repository_creates_branch(self):
        self.run_script("""
$ brz colo-init local
$ brz init-repo repo
$ brz init repo/not_trunk
$ cd repo/not_trunk
$ brz ci -m 1 --unchanged
$ cd ../../local
$ brz colo-sync-from --repository ../repo
$ brz colo-branches
  not_trunk
* trunk
""")

    def test_sync_from_repository_not_remembered(self):
        self.run_script("""
$ brz colo-init local
$ brz init-repo repo
$ cd local
$ brz colo-sync-from --repository ../repo
$ brz colo-sync-from
2>brz: ERROR: No synchronization location known or specified.
""")



class TestColoSyncTo(ScriptTestCase):

    def test_sync(self):
        self.run_script("""
$ brz colo-init remote
$ brz colo-init local
$ cd local
$ brz colo-sync-to ../remote
2>Updating colo:../remote:trunk from trunk.
2>No new revisions or tags to push.
""")

    def test_sync_nontrivial(self):
        self.run_script("""
$ brz colo-init remote
$ brz colo-init local
$ cd local
$ echo content > file
$ brz add
$ brz ci -m 'first revision'
$ brz colo-sync-to ../remote
2>Updating colo:../remote:trunk from trunk.
2>Pushed up to revision 1.
$ cd ../remote
$ brz revno
1
""")

    def test_sync_to_empty(self):
        self.run_script("""
$ mkdir branch
$ brz colo-init workspace
$ cd workspace
$ brz colo-sync-to ../branch
2>brz: ERROR: No colocated workspace in .../branch
""")

    def test_sync_to_standalone(self):
        self.run_script("""
$ brz colo-init local
$ brz init standalone
$ cd local
$ brz colo-sync-to ../standalone
2>brz: ERROR: No colocated workspace in ../standalone
""")

    def test_sync_to_no_trees(self):
        self.run_script("""
$ brz colo-init --no-tree remote
$ brz colo-init local
$ cd local
$ brz colo-sync-to ../remote
2>Updating colo:../remote:trunk from trunk.
2>No new revisions or tags to push.
$ cd ../remote
$ brz colo-branches
  trunk
""")

    def test_sync_to_creates_branch(self):
        self.run_script("""
$ brz colo-init remote
$ brz colo-init local
$ cd local
$ brz colo-branch other
$ brz colo-sync-to ../remote
2>Updating colo:../remote:other from other.
2>Created new branch.
2>Updating colo:../remote:trunk from trunk.
2>No new revisions or tags to push.
$ cd ../remote
$ brz colo-branches
  other
* trunk
""")

    def test_sync_to_diverged(self):
        self.run_script("""
$ brz colo-init remote
$ brz colo-init local
$ cd remote
$ echo content > file
$ brz add
$ brz ci -m 'a revision'
$ cd ../local
$ echo stuff > file
$ brz add
$ brz ci -m 'a separate revision'
$ brz colo-sync-to ../remote
2>Updating colo:../remote:trunk from trunk.
2>brz: ERROR: These branches have diverged. ...
""")

    def test_sync_to_repository(self):
        self.run_script("""
$ brz colo-init local
$ brz init-repo repo
$ brz init repo/trunk
$ cd local
$ echo content > file
$ brz add
$ brz ci -m 'a revision in colocated branch'
$ brz colo-sync-to --repository ../repo
2>Updating colo:../repo:trunk from trunk.
2>All changes applied successfully.
2>Pushed up to revision 1.
""")

    def test_sync_to_repository_no_argument(self):
        self.run_script("""
$ brz colo-init local
$ brz init-repo repo
$ brz init repo/trunk
$ cd local
$ echo content > file
$ brz add
$ brz ci -m 'a revision in colocated branch'
$ brz colo-sync-to ../repo
$ cat ../repo/trunk/.bzr/README
$ cat ../repo/trunk/file
2>../repo/trunk/file: No such file or directory
""")

    def test_sync_to_repository_creates_branch(self):
        self.run_script("""
$ brz colo-init local
$ brz init-repo repo
$ cd local
$ brz ci -m 1 --unchanged
$ brz colo-sync-to --repository ../repo
$ cat ../repo/trunk/.bzr/README
""")

    def test_sync_to_repository_not_remembered(self):
        self.run_script("""
$ brz colo-init local
$ brz init-repo repo
$ cd local
$ brz ci -m 1 --unchanged
$ brz colo-sync-to --repository ../repo
$ brz colo-sync-to
2>brz: ERROR: No synchronization location known or specified.
""")


class TestColoSyncLocations(ScriptTestCase):

    def test_set_locations(self):
        self.run_script('$ brz colo-init')
        w = ColocatedWorkspace()
        c = w.get_config()
        self.assertEqual(c.get_sync_from_location(), None)
        self.assertEqual(c.get_sync_to_location(), None)
        c.set_sync_from_location('location')
        c.set_sync_to_location('other_location')
        self.assertEqual(c.get_sync_from_location(), 'location')
        self.assertEqual(c.get_sync_to_location(), 'other_location')

    def test_set_location_none(self):
        self.run_script('$ brz colo-init')
        w = ColocatedWorkspace()
        c = w.get_config()
        c.set_sync_from_location(None)
        self.assertEqual(c.get_sync_from_location(), None)
        c.set_sync_to_location(None)
        self.assertEqual(c.get_sync_to_location(), None)

    def test_sync_from_default_location(self):
        self.run_script("""
$ brz colo-init remote
$ brz colo-init local
$ cd local
$ brz colo-sync-from ../remote
$ brz colo-sync-from
2>Updating branch trunk from .../remote/:trunk.
No revisions or tags to pull.
""")

    def test_sync_from_remembered_location(self):
        self.run_script("""
$ brz colo-init remote1
$ brz colo-init remote2
$ brz colo-init local
$ cd local
$ brz colo-sync-from ../remote1
2>Updating branch trunk from ...:trunk.
No revisions or tags to pull.
$ brz colo-sync-from --remember ../remote2
$ brz colo-sync-from
2>Updating branch trunk from ...remote2/:trunk.
""")

    def test_sync_from_no_default(self):
        self.run_script("""
$ brz colo-init
$ brz colo-sync-from
2>brz: ERROR: No synchronization location known or specified.
""")

    def test_sync_to_default_location(self):
        self.run_script("""
$ brz colo-init remote
$ brz colo-init local
$ cd local
$ brz colo-sync-to ../remote
$ brz colo-sync-to
2>Updating colo:.../remote/:trunk from trunk.
2>No new revisions or tags to push.
""")

    def test_sync_to_remembered_location(self):
        self.run_script("""
$ brz colo-init remote1
$ brz colo-init remote2
$ brz colo-init local
$ cd local
$ brz colo-sync-to ../remote1
2>Updating colo:.../remote1:trunk from trunk.
2>No new revisions or tags to push.
$ brz colo-sync-to --remember ../remote2
$ brz colo-sync-to
2>Updating colo:.../remote2/:trunk from trunk.
2>No new revisions or tags to push.
""")

    def test_sync_to_no_default(self):
        self.run_script("""
$ brz colo-init
$ brz colo-sync-to
2>brz: ERROR: No synchronization location known or specified.
""")
